package fr.plnech.dunbar.ui

import android.content.ContentUris
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.os.Bundle
import android.provider.ContactsContract
import android.view.Menu
import android.view.MenuItem
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.core.app.NotificationManagerCompat
import androidx.recyclerview.widget.LinearLayoutManager
import fr.plnech.dunbar.R
import fr.plnech.dunbar.data.Messages
import fr.plnech.dunbar.model.Friend
import fr.plnech.dunbar.notif.FriendReminder
import fr.plnech.dunbar.plural
import kotlinx.android.synthetic.main.activity_friends.*
import kotlinx.android.synthetic.main.content_friends.*
import java.io.IOException


class FriendsActivity : AppCompatActivity() {

    private lateinit var messages: Messages
    private val friends = mutableListOf<Friend>()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_friends)
        setSupportActionBar(toolbar)

//        messages = Messages(applicationContext)

        fab.setOnClickListener { view ->
            reloadFriends()
        }

        reloadFriends()
        notifyFriend()
    }

    private fun notifyFriend() {
        val idNotif = 0
        with(NotificationManagerCompat.from(this)) {
            val notification =
                FriendReminder(applicationContext).createNotification(friends[0], this@FriendsActivity)
            notification?.let {
                notify(idNotif, notification)
            }
        }
    }

    override fun onCreateOptionsMenu(menu: Menu): Boolean {
        // Inflate the menu; this adds items to the action bar if it is present.
        menuInflater.inflate(R.menu.menu_main, menu)
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        return when (item.itemId) {
            R.id.action_settings -> true
            else -> super.onOptionsItemSelected(item)
        }
    }

    private fun reloadFriends() {
        fetchFriends()
        displayFriends()
//        fetchMessages()
    }

    private fun fetchMessages() {
        messages.fetchAll()
        Toast.makeText(
            this, "${messages.sent.size} messages sent, " +
                    "${messages.inbox.size} messages received.", Toast.LENGTH_SHORT
        ).show()
    }

    private fun fetchFriends() {
        val newFriends = mutableListOf<Friend>()

        contentResolver.query(
            ContactsContract.CommonDataKinds.Contactables.CONTENT_URI,
            null,
            null,
            null,
            null
        )?.let {
            while (it.moveToNext()) {
                mutableMapOf<String, String?>().also { map ->
                    val indexId = it.getColumnIndex(ContactsContract.Contacts._ID)
                    val id = it.getLong(indexId)
                    val photo: Bitmap? = getPhoto(id)

                    it.columnNames.forEach { name ->
                        val index = it.getColumnIndex(name)
                        map[name] = it.getString(index)
                    }
                    newFriends.add(Friend(map, photo))
                }
            }
            it.close()

            friends.clear()
            friends.addAll(newFriends)
        }

        friends.sortBy { it.timesContacted }
    }

    private fun displayFriends() {
        val adapter = FriendsAdapter(friends)

        friendsList.layoutManager = LinearLayoutManager(this)
        friendsList.setHasFixedSize(true)
        friendsList.adapter = adapter
//        friendsList.setOnItemClickListener { parent, view, position, id ->
//            val contact: String = adapter.getItem(position)?.toString() ?: "NONE"
//
//                Snackbar.make(friendsList, contact, Snackbar.LENGTH_INDEFINITE).show()
//            Toast.makeText(this, contact, Toast.LENGTH_LONG).show()
//        }

        val nbFriends = adapter.itemCount
        welcomeTitle.text = "$nbFriends ${"friend".plural(nbFriends)} on Dunbar"
    }

    private fun getPhoto(id: Long): Bitmap? {
        var photo: Bitmap? = null

        try {
            val inputStream = ContactsContract.Contacts.openContactPhotoInputStream(
                contentResolver,
                ContentUris.withAppendedId(
                    ContactsContract.Contacts.CONTENT_URI,
                    java.lang.Long.valueOf(id)
                )
            )

            if (inputStream != null) {
                photo = BitmapFactory.decodeStream(inputStream)
            }

            inputStream?.close()
        } catch (e: IOException) {
            e.printStackTrace()
        }
        println("Photo for $id: $photo")
        return photo
    }
}
